package ac.essex.ooechs.adaboost;

import java.io.Serializable;

/**
 * Allows the output of an Adaboost problem to be tested in a lightweight, standalone
 * class.
 *
 * @author Olly Oechsle, University of Essex, Date: 21-Sep-2007
 * @version 1.1
 */
public class AdaBoostSolution implements Serializable {

    protected AdaBoostLearner[] h;
    protected double[] b;
    protected int cursor;

    protected AdaBoostSolution(int numHypotheses) {
        h = new AdaBoostLearner[numHypotheses];
        b = new double[numHypotheses];
        cursor = 0;
    }

    protected void addHypothesis(AdaBoostLearner hypothesis, double betaValue) {
        h[cursor] = hypothesis;
        b[cursor] = betaValue;
        cursor++;
    }

    public int classify(AdaBoostSample sample, Object data) {

        // each weak learner is run, whatever label that learner finds is given a value
        // which is the log of the learner's beta value, so learners with low error rates will
        // return a higher value. Whichever label has the highest score is the one that gets returned.

        // create an array that can accommodate 100 classIDs
        double[] classSums = new double[100];

        // the label to be outputted
        int mostLikelyLabel = -1;

        // the highest score for any label
        double highestScore = Double.MIN_VALUE;

        for (int t = 0; t < h.length; t++) {

            // in case h does not contain T classifiers
            if (h[t] == null) continue;

            // Get the learner's output: a class label
            int label = h[t].classify(sample, data);

            // Add a value to the sum associated with this label
            classSums[label] += Math.log10(1 / b[t]);

            // if this label has the highest score,
            // set this label as the one to be returned
            if (classSums[label] > highestScore) {
                highestScore = classSums[label];
                mostLikelyLabel = label;
            }

        }

        // return the label that has the highest score
        return mostLikelyLabel;

    }

}
